import { COMMAND } from './build-context';
import { PWAManifest } from './pwa';
export interface WebpackFilenames {
    chunk?: ({ isDev }: {
        isDev: boolean;
    }) => string;
    file?: ({ isDev }: {
        isDev: boolean;
    }) => string;
    sourceMap?: ({ isDev }: {
        isDev: boolean;
    }) => string;
}
/**
 * GENX CLI configuration
 * Loaded from genx.config.{js,json,ts} or genx key in package.json in working directory
 */
export interface Config {
    app?: {
        /**
         * Root application element name (must include hyphen)
         */
        rootElement?: string;
    };
    /**
     * Override default builder for a given CLI command (dev/build/analyze/test).
     * A given builder typically supports a subset of all possible commands (e.g. a test runner won't handle analyze command).
     * If you use an incompatible builder, you'll get an unrecognized command error.
     * @defaultValue
     * ```
     * {
     *   default: 'webpack',
     *   dev: 'vite',
     *   lint: 'eslint-stylelint',
     *   test: 'uvu-playwright',
     * }
     * ```
     * @example
     * ```
     * "builder": {
     *   "build": "ts",
     *   "dev": "ts"
     * }
     * ```
     */
    builder?: {
        [index in COMMAND | 'default']: string;
    };
    /**
     * Controls whether dist folder should be cleared out before the build.
     * Can be overriden using `GENX_NO_CLEAN` environment variable.
     * @defaultValue true
     */
    clean?: boolean;
    /**
     * Static assets to copy during the build.
     * @defaultValue ['src/**\/*.{css,jpg,png,svg,mp4,ogg,webm}']
     */
    copyAssets?: string[];
    /**
     * Override default entry point
     * Paths are resolved relative to src folder
     * @example
     * ```
     * "entry": {
     *   "electron": "electron.main.ts"
     * }
     * ```
     */
    entry?: string | {
        [index: string]: string;
    };
    /**
     * Environment variables to expose in UI code. Regex patterns allowed.
     * @defaultValue `['API_HOST', 'DEFAULT_ORGANISATION', 'DEFAULT_USER', 'DEFAULT_PASSWORD', 'ENABLE_SSO', 'GENX.*', 'GENX_*', 'FORCE_HTTP', 'SOCKET_EXT']
     */
    env?: string[];
    /**
     * Exclude modules from the output bundle.
     * @remark This setting has no effect on `ts-builder`
     * @defaultValue `index.ts`
     * @example
     * ```
     * "externals": {
     *   "@ag-grid-community/core": "agGrid"
     * }
     * ```
     */
    externals?: {
        [index: string]: string;
    };
    /**
     * Module federation options.
     */
    federation?: {
        /**
         * Federation dashboard URL
         */
        dashboardURL?: string;
        /**
         * Federation publish version
         * Defaults to version in package.json.
         */
        publishVersion?: string;
        /**
         * Remote federated modules
         * @example
         * ```
         * remotes: ["@genesislcap/foundation-zero"]
         * ```
         */
        remotes?: {
            [index: string]: string;
        };
        /**
         * Shared federated modules
         * @example
         * ```
         * shared: ["@microsoft/fast-components", "@microsoft/fast-element", "@microsoft/fast-foundation"]
         * ```
         */
        shared?: {
            [index: string]: string | {
                singleton: boolean;
                requiredVersion: string;
            };
        };
        /**
         * Exposed entry points
         * @example
         * ```
         * exposes: {
         *   "./index": "./src/index.federated.ts",
         *   "./utils": "./src/utils/index.ts"
         * }
         * ```
         */
        exposes?: {
            [index: string]: string;
        };
    };
    /**
     * HTTP polling configuration.
     * @example
     * ```
     * http: {
     *  "POLLING_MAP": {
     *    "EVENT_AMEND_USER": {
     *     "TEMP_FREQUENCY": 400,
     *      "AMOUNT_OF_POLLS": 5,
     *      "QUERIES": [
     *        "ALL_USERS"
     *      ]
     *    }
     *  },
     *  "POLLING_INTERVAL_MAP": {
     *    "ALL_TRADES": 4000,
     *    "ALL_USERS": 4000,
     *    "ALL_PROCESSES_STATUS": 5000
     *  },
     *  "POLLING_FREQUENCY": 4000
     * }
     * ```
     */
    http?: object;
    pwa?: PWAManifest;
    serviceWorker?: {
        worker: string;
        registration: string;
        assetName?: string;
        exclude?: Array<string | RegExp | ((arg0: any) => boolean)>;
        maximumFileSizeToCache?: number;
    };
    webpack?: {
        node?: false | {
            [index: string]: boolean | string;
        };
        /**
         * Customize bundle filenames.
         *
         * @note Be careful when using non-hashed based filenames in production
         * as most browsers will cache the asset and not detect the changes on first load.
         *
         * @see https://webpack.js.org/configuration/output/#outputfilename
         * @example
         * ```js
         * filenames: {
         *   chunk: ({ isDev }) => (isDev ? '[name].js' : '[id].[contenthash].js')
         * }
         * ```
         */
        filenames?: WebpackFilenames;
        /**
         * Customize Webpack optimization settings.
         *
         * @see https://webpack.js.org/configuration/optimization/
         */
        optimization?: ({ isDev }: {
            isDev: boolean;
        }) => unknown;
        /**
         * Set Webpack deployment target.
         *
         * @see https://webpack.js.org/concepts/targets/
         */
        target?: string | false | string[];
        /**
         * Dev server middleware writeToDisk toggle.
         *
         * @remarks
         * Useful during development when watching packages for changes that are bundled with webpack.
         *
         * @example
         * ```json
         * "genx": {
         *     "app": {...},
         *     "webpack": {
         *       "writeToDisk": true
         *     }
         *   },
         * ```
         *
         * @see https://webpack.js.org/configuration/dev-server/#devserverdevmiddleware
         */
        writeToDisk?: boolean;
    };
}
//# sourceMappingURL=config.d.ts.map