"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveDefineConfig = exports.setEnvVars = exports.loadEnvFiles = exports.setNodeEnv = void 0;
const tslib_1 = require("tslib");
const node_path_1 = require("node:path");
const consola_1 = tslib_1.__importDefault(require("consola"));
const dotenv_1 = tslib_1.__importDefault(require("dotenv"));
const setNodeEnv = (targetEnv) => {
    const currentEnv = process.env.NODE_ENV;
    if (currentEnv && currentEnv !== targetEnv) {
        consola_1.default.warn(`Changing \`NODE_ENV\` from \`${currentEnv}\` to \`${targetEnv}\`.`);
    }
    process.env.NODE_ENV = targetEnv;
};
exports.setNodeEnv = setNodeEnv;
const resolveEnvValue = (ctx, key, value) => {
    // `-e `FOO` case
    // 1. check package.json config
    // 2. fall back to a truthy value (shortcut for setting env var flags where exact value isn't important as long as it's truthy e.g. -e CI)
    // we don't check process.env because there it'd mean setting env variable to the same value it already has
    if (!value) {
        return ctx.pkg.config && ctx.pkg.config[key] !== undefined ? ctx.pkg.config[key] : '1';
    }
    // `-e FOO=ABC` case
    // 1. check package.json config
    if (ctx.pkg.config && ctx.pkg.config[value] !== undefined) {
        return ctx.pkg.config[value];
    }
    // 2. check process.env
    if (process.env[value] !== undefined) {
        return process.env[value];
    }
    // 3. fall back to provided value
    return value;
};
const loadEnvFiles = (ctx) => {
    const useWorkspaceDir = ctx.dirs.cwd && ctx.dirs.workspace && ctx.dirs.cwd !== ctx.dirs.workspace;
    const path = [
        useWorkspaceDir && (0, node_path_1.resolve)(ctx.dirs.workspace, '.env'),
        useWorkspaceDir && (0, node_path_1.resolve)(ctx.dirs.workspace, '.env.local'),
        (0, node_path_1.resolve)(ctx.dirs.cwd, '.env'),
        (0, node_path_1.resolve)(ctx.dirs.cwd, '.env.local'),
    ].filter(Boolean);
    dotenv_1.default.config({
        debug: ctx.cli.options.logLevel === 'verbose',
        // if the same variable occurs in multiple files, the last loaded file wins
        override: true,
        path,
    });
};
exports.loadEnvFiles = loadEnvFiles;
const setEnvVars = (ctx) => {
    (0, exports.loadEnvFiles)(ctx);
    const { env = '' } = ctx.cli.options;
    const envCombined = Array.isArray(env) ? env.join(',') : env;
    const vars = envCombined.split(',');
    vars.forEach((v) => {
        const [keyRaw, valueRaw] = v.split('=');
        const key = (keyRaw || '').trim();
        const value = (valueRaw || '').trim();
        if (key) {
            const resolved = resolveEnvValue(ctx, key, value);
            consola_1.default.info(`Setting \`${key}\` env variable to \`${resolved}\`.`);
            process.env[key] = resolved;
        }
    });
};
exports.setEnvVars = setEnvVars;
// Given env variable name patterns and HTTP config, return a define plugin config
const resolveDefineConfig = (patterns, httpConfig, builder) => Object.entries(process.env)
    .filter(([key]) => patterns.some((pattern) => key.match(pattern)))
    .reduce((acc, [key, value]) => {
    acc[key] = JSON.stringify(value);
    return acc;
}, {
    NODE_ENV: JSON.stringify(process.env.NODE_ENV),
    HTTP_CONFIG: JSON.stringify(httpConfig || ''),
    BUILDER: JSON.stringify(builder !== null && builder !== void 0 ? builder : ''),
});
exports.resolveDefineConfig = resolveDefineConfig;
