"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateVersions = exports.updatePackageJSON = exports.getRawPkg = exports.getRangeSpecifier = exports.getLatestVersion = exports.loadPackageJSONFiles = exports.loadPackageJSON = exports.resolveBin = exports.tryRequire = void 0;
const tslib_1 = require("tslib");
const node_child_process_1 = require("node:child_process");
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const consola_1 = tslib_1.__importDefault(require("consola"));
const jiti_1 = tslib_1.__importDefault(require("jiti"));
const pkg_types_1 = require("pkg-types");
// Attempt to load id.{js,json,ts} file
const tryRequire = (id, rootDir = process.cwd()) => {
    const _require = (0, jiti_1.default)(rootDir, { interopDefault: true, esmResolve: true });
    try {
        return _require(id);
    }
    catch (error) {
        if (error.code !== 'MODULE_NOT_FOUND') {
            consola_1.default.error(`Error trying import ${id} from ${rootDir}`, error);
        }
        return null;
    }
};
exports.tryRequire = tryRequire;
const resolveBin = (bin, module = bin) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const modulePath = require.resolve(module);
    const dir = (0, node_path_1.dirname)(yield (0, pkg_types_1.resolvePackageJSON)(modulePath));
    const json = yield (0, pkg_types_1.readPackageJSON)(modulePath);
    const path = typeof json.bin === 'string' ? json.bin : json.bin[bin];
    if (!path || !dir) {
        throw new Error(`Could not resolve ${bin} / ${module}: ${dir} ${path}`);
    }
    return (0, node_path_1.resolve)(dir, path);
});
exports.resolveBin = resolveBin;
const loadPackageJSON = (dir) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const pkg = (yield (0, pkg_types_1.readPackageJSON)(dir)) || {};
    const nameParts = (pkg.name || '').split('/');
    // @scope/name -> name
    // name -> name
    pkg.nameUnscoped = nameParts.pop();
    // @scope/name -> @scope
    // name -> undefined
    pkg.nameScope = nameParts.shift();
    return pkg;
});
exports.loadPackageJSON = loadPackageJSON;
const loadPackageJSONFiles = (packages = []) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    return Promise.all(packages.map((p) => {
        const dir = (0, node_path_1.dirname)(require.resolve(p));
        return (0, exports.loadPackageJSON)(dir);
    }));
});
exports.loadPackageJSONFiles = loadPackageJSONFiles;
/*
Given package name and range (e.g. '^14.0.1', '~14.0.1' or 'latest', return latest matching version)

Registry response varies depending on whether a single or multiple versions are available.
We return the single version which is the latest available.

Response if a single version is available:
'14.41.1'

Response if multiple versions available:
@genesislcap/genx@14.41.0 '14.41.0'
@genesislcap/genx@14.41.1 '14.41.1'
*/
const getLatestVersion = (name, range = 'latest', verbose = true) => {
    let result;
    try {
        result = (0, node_child_process_1.execSync)(`npm info ${name}@${range} version`, {
            stdio: ['pipe', 'pipe', 'ignore'],
        })
            .toString('utf8')
            .trim()
            .split('\n')
            .pop()
            .split(' ')
            .pop()
            .replace(/'/g, '');
    }
    catch (e) {
        if (verbose) {
            consola_1.default.error(`Could not fetch ${name} version ${range} information from the registry`);
        }
    }
    return result;
};
exports.getLatestVersion = getLatestVersion;
// given package version (^14.0.0, ~14.0.0), return range specifier (^, ~)
const getRangeSpecifier = (version) => {
    const match = version.toString().match(/^[><=^~*]+/);
    return (match && match[0]) || '';
};
exports.getRangeSpecifier = getRangeSpecifier;
// Return package.json without additional fields added by us
const getRawPkg = (ctx) => {
    const _a = ctx.pkg, { nameUnscoped, nameScope } = _a, pkg = tslib_1.__rest(_a, ["nameUnscoped", "nameScope"]);
    return pkg;
};
exports.getRawPkg = getRawPkg;
const updatePackageJSON = (pkg) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const pkgJSONPath = yield (0, pkg_types_1.resolvePackageJSON)();
    try {
        (0, node_fs_1.writeFileSync)(pkgJSONPath, `${JSON.stringify(pkg, null, 2)}\n`);
    }
    catch (e) {
        consola_1.default.error(`Error updating package.json: ${pkgJSONPath}`, e);
    }
});
exports.updatePackageJSON = updatePackageJSON;
const updateVersionType = (deps, newVersion, exclude, include = ['@genesislcap']) => {
    const updates = [];
    Object.entries(deps || {}).forEach(([name, version = '']) => {
        if (include.some((pattern) => name.includes(pattern)) &&
            exclude.every((pattern) => !name.includes(pattern))) {
            const available = newVersion || (0, exports.getLatestVersion)(name, version);
            if (!available) {
                return;
            }
            const rangeSpecifier = (0, exports.getRangeSpecifier)(version);
            const versionNew = `${rangeSpecifier}${available}`;
            if (version !== versionNew) {
                updates.push({
                    name,
                    from: version,
                    to: versionNew,
                });
                deps[name] = versionNew;
            }
        }
    });
    return updates;
};
const updateVersions = (pkg, newVersion, exclude, include = ['@genesislcap']) => {
    const deps = updateVersionType(pkg.dependencies, newVersion, exclude, include);
    const devDeps = updateVersionType(pkg.devDependencies, newVersion, exclude, include);
    const peerDeps = updateVersionType(pkg.peerDependencies, newVersion, exclude, include);
    return [...deps, ...devDeps, ...peerDeps];
};
exports.updateVersions = updateVersions;
