"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getData = exports.getPromptValidators = exports.getPromptDefaults = exports.validateSeedData = exports.runPrompts = void 0;
const tslib_1 = require("tslib");
const node_path_1 = require("node:path");
const consola_1 = tslib_1.__importDefault(require("consola"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const console_1 = require("./console");
const runPrompts = (prompts, prevAns = {}) => tslib_1.__awaiter(void 0, void 0, void 0, function* () { return prompts(inquirer_1.default, prevAns); });
exports.runPrompts = runPrompts;
const validateSeedData = (data, validators, defaults) => {
    const errors = [];
    Object.keys(data).forEach((key) => {
        const value = data[key];
        if (typeof value === 'undefined') {
            errors.push({ message: `${key} is undefined` });
        }
        else if (value === '' && defaults[key] !== '') {
            errors.push({ message: `${key} is empty` });
        }
        else if (validators[key]) {
            const isValid = validators[key](value);
            if (isValid !== true) {
                errors.push({ message: `${key} is invalid: ${(0, console_1.bold)(value)} ${isValid}` });
            }
        }
    });
    return errors;
};
exports.validateSeedData = validateSeedData;
const getPromptInputs = (prompts) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    // disable any prompt messages
    const _log = console.log;
    console.log = () => { };
    const inputs = [];
    const mockInquirer = {
        prompt: (promptInputs) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
            inputs.push(...promptInputs);
            return {};
        }),
    };
    yield prompts(mockInquirer);
    // re-enable prompt messages
    console.log = _log;
    return inputs;
});
const getPromptDefaults = (prompts) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const inputs = yield getPromptInputs(prompts);
    const defaults = inputs.reduce((acc, value) => {
        let addDefault;
        if (typeof value.when === 'function') {
            addDefault = value.when(acc);
        }
        else if (typeof value.when === 'undefined') {
            addDefault = true;
        }
        else {
            addDefault = value.when;
        }
        if (addDefault) {
            acc[value.name] = value.default;
        }
        return acc;
    }, {});
    return defaults;
});
exports.getPromptDefaults = getPromptDefaults;
const getPromptValidators = (prompts) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const inputs = yield getPromptInputs(prompts);
    const validators = inputs.reduce((acc, value) => {
        acc[value.name] = value.validate;
        return acc;
    }, {});
    return validators;
});
exports.getPromptValidators = getPromptValidators;
// process CLI prompt params
const getCLIParams = (persistedAnswers, params, defaults, dir, subdir, skipOptionalPrompts, seedMeta, details, validate) => {
    const base = skipOptionalPrompts ? defaults : {};
    const cliParams = Object.assign(Object.assign(Object.assign(Object.assign({}, base), persistedAnswers), params), { appName: (0, node_path_1.basename)(dir), pkgName: (0, node_path_1.basename)(dir), directory: dir, appSeed: {
            type: seedMeta.local ? 'local' : 'remote',
            name: (0, node_path_1.basename)(seedMeta.seed || ''),
            description: details.description,
            version: details.version,
            repo: seedMeta.repo,
            artifact: seedMeta.artifact,
            ref: seedMeta.ref,
            subdir: seedMeta.subdir,
            date: new Date(),
        } });
    const validationErrors = validate(cliParams);
    if (validationErrors.length) {
        consola_1.default.error('Validation errors detected:');
        validationErrors.forEach((error) => consola_1.default.log(`  * ${error.message}`));
        process.exit(1);
    }
    return cliParams;
};
// get combined prompt answers (CLI params / defaults / user interaction)
const getData = (persistedAnswers, prompts, details, defaults, validate, params, dir, skipOptionalPrompts, seedMeta) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    // process CLI params
    const cliParams = getCLIParams(persistedAnswers, params, defaults, dir, seedMeta.subdir, skipOptionalPrompts, seedMeta, details, validate);
    const data = yield (0, exports.runPrompts)(prompts, cliParams);
    const combined = Object.assign(Object.assign({}, cliParams), data);
    return combined;
});
exports.getData = getData;
