import type { ServerOptions } from 'http-proxy';
/**
 * @public
 */
export declare const getPort: (packageJson: any) => number;
/**
 * @public
 */
export declare const convertWsEndpointToHTTP: (endpoint?: string, socketExt?: string) => string;
/**
 * Config to create a development proxy for a specific path.
 *
 * @public
 */
export type DevProxyConfig = {
    target?: string;
    path?: string;
    options?: ServerOptions;
};
/**
 * Config to create development proxies for an array of paths.
 *
 * @public
 */
export type DevProxiesConfig = Omit<DevProxyConfig, 'path'> & {
    paths?: string[];
};
/**
 * Object containing entries for each proxy keyed by path.
 *
 * @public
 */
export type DevProxies = Record<string, ServerOptions> & {};
/**
 * Create development proxy for a specific path.
 *
 * @example
 * ```ts
 * const proxy = createDevProxy({ path: '/sm', target: process.env.API_HOST, options: {...} });
 * ```
 *
 * @remarks
 * Applicable when running in dev mode and a valid target and path is set.
 *
 * @privateRemarks
 * It is important to guard for the case where neither target nor path is set by returning undefined.
 * Proxies with empty string targets will cause blow ups in vite, whereas webpack seems to tolerate them.
 *
 * @param config - A {@link DevProxyConfig}.
 * @returns A {@link DevProxies} object or undefined.
 *
 * @public
 */
export declare const createDevProxy: (config: DevProxyConfig) => DevProxies | undefined;
/**
 * Create development proxies for an array of paths.
 *
 * @example Create default development proxies
 * ```ts
 * const proxy = createDevProxies();
 * ```
 *
 * @example Create custom development proxies
 * ```ts
 * const proxy = createDevProxies({ paths: ['/foo', '/bar'], target: 'http://mydomain.com' });
 * ```
 *
 * @remarks
 * `process.env.API_HOST` will be used as the target if `config.target` is not set. Undefined will be returned if
 * neither are available.
 *
 * @privateRemarks
 * It is important to guard for the case where neither target nor API_HOST are set by returning undefined.
 * Proxies with empty string targets will cause blow ups in vite, whereas webpack seems to tolerate them.
 *
 * @param config - An optional {@link DevProxiesConfig}.
 * @returns A {@link DevProxies} object or undefined.
 *
 * @public
 */
export declare const createDevProxies: (config?: DevProxiesConfig) => DevProxies | undefined;
export declare const createDevProxiesArray: (config?: DevProxiesConfig) => ServerOptions[];
//# sourceMappingURL=proxy.d.ts.map