"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDevProxiesArray = exports.createDevProxies = exports.createDevProxy = exports.convertWsEndpointToHTTP = exports.getPort = void 0;
const tslib_1 = require("tslib");
const consola_1 = tslib_1.__importDefault(require("consola"));
const portfinder_sync_1 = tslib_1.__importDefault(require("portfinder-sync"));
/**
 * @public
 */
const getPort = (packageJson) => { var _a; return portfinder_sync_1.default.getPort(((_a = packageJson.config) === null || _a === void 0 ? void 0 : _a.PORT) || process.env.PORT || 9000); };
exports.getPort = getPort;
/**
 * @public
 */
const convertWsEndpointToHTTP = (endpoint = '', socketExt = '') => {
    const protocolUpdate = endpoint.replace(/^ws(s)?:\/\//, 'http$1://');
    return socketExt ? protocolUpdate.replace(`/${socketExt}`, '') : protocolUpdate;
};
exports.convertWsEndpointToHTTP = convertWsEndpointToHTTP;
/**
 * Create development proxy for a specific path.
 *
 * @example
 * ```ts
 * const proxy = createDevProxy({ path: '/sm', target: process.env.API_HOST, options: {...} });
 * ```
 *
 * @remarks
 * Applicable when running in dev mode and a valid target and path is set.
 *
 * @privateRemarks
 * It is important to guard for the case where neither target nor path is set by returning undefined.
 * Proxies with empty string targets will cause blow ups in vite, whereas webpack seems to tolerate them.
 *
 * @param config - A {@link DevProxyConfig}.
 * @returns A {@link DevProxies} object or undefined.
 *
 * @public
 */
const createDevProxy = (config) => {
    var _a;
    if (!config.target || !config.path) {
        return undefined;
    }
    const { options, path, target } = config;
    return {
        [path]: Object.assign(Object.assign({ target, secure: false, changeOrigin: true, cookieDomainRewrite: 'localhost', ws: true }, (options !== null && options !== void 0 ? options : {})), { headers: Object.assign({ origin: target }, ((_a = options === null || options === void 0 ? void 0 : options.headers) !== null && _a !== void 0 ? _a : {})) }),
    };
};
exports.createDevProxy = createDevProxy;
/**
 * Create development proxies for an array of paths.
 *
 * @example Create default development proxies
 * ```ts
 * const proxy = createDevProxies();
 * ```
 *
 * @example Create custom development proxies
 * ```ts
 * const proxy = createDevProxies({ paths: ['/foo', '/bar'], target: 'http://mydomain.com' });
 * ```
 *
 * @remarks
 * `process.env.API_HOST` will be used as the target if `config.target` is not set. Undefined will be returned if
 * neither are available.
 *
 * @privateRemarks
 * It is important to guard for the case where neither target nor API_HOST are set by returning undefined.
 * Proxies with empty string targets will cause blow ups in vite, whereas webpack seems to tolerate them.
 *
 * @param config - An optional {@link DevProxiesConfig}.
 * @returns A {@link DevProxies} object or undefined.
 *
 * @public
 */
const createDevProxies = (config = {}) => {
    var _a;
    if (!config.target && !process.env.API_HOST) {
        return undefined;
    }
    const socketExt = process.env.SOCKET_EXT || 'gwf';
    const paths = Array.isArray(config.paths) ? config.paths : ['/sm', '/sso', `/${socketExt}`];
    const target = (_a = config.target) !== null && _a !== void 0 ? _a : (0, exports.convertWsEndpointToHTTP)(process.env.API_HOST, socketExt);
    const defaultConfig = Object.assign(Object.assign({}, config), { target });
    const proxies = paths.reduce((prev, curr) => (Object.assign(Object.assign({}, prev), (0, exports.createDevProxy)(Object.assign(Object.assign({}, defaultConfig), { path: curr })))), {});
    if (Object.keys(proxies).length) {
        consola_1.default.info('Dev Proxies:');
        console.table(proxies);
    }
    return proxies;
};
exports.createDevProxies = createDevProxies;
const createDevProxiesArray = (config = {}) => {
    const proxiesObject = (0, exports.createDevProxies)(config);
    const proxiesArray = proxiesObject
        ? Object.entries(proxiesObject).map(([path, proxyConfig]) => (Object.assign(Object.assign({}, proxyConfig), { path })))
        : [];
    return proxiesArray;
};
exports.createDevProxiesArray = createDevProxiesArray;
