"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchSeed = exports.getSeedDirs = exports.resolveNPMSeed = exports.resolveGitSeed = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const consola_1 = require("consola");
const get_npm_tarball_url_1 = tslib_1.__importDefault(require("get-npm-tarball-url"));
const giget_1 = require("giget");
const archive_1 = require("./archive");
const console_1 = require("./console");
const fs_1 = require("./fs");
const npm_1 = require("./npm");
const seed_meta_1 = require("./seed.meta");
const seed_processing_1 = require("./seed.processing");
const resolveGitSeed = (seed = '', ref = 'main', subdir = '', provider = 'gh') => {
    const parts = seed.split('/');
    const repo = parts.pop() || 'blank-app-seed';
    const user = parts.pop() || 'genesiscommunitysuccess';
    const dir = subdir ? `/${subdir}` : '';
    return {
        repo: `${user}/${repo}`,
        ref,
        // Giget-compatible template ID
        id: `${provider}:${user}/${repo}${dir}#${ref}`,
    };
};
exports.resolveGitSeed = resolveGitSeed;
const resolveNPMSeed = (seed = '', ref = 'latest', useDefaultNPMScope = true) => {
    const parts = seed.split('/');
    const moduleName = parts.pop() || 'blank-app-seed';
    const scope = parts.pop() || (useDefaultNPMScope ? 'genesislcap' : '');
    const scopeFormatted = scope ? `@${scope}/` : '';
    const artifact = `${scopeFormatted}${moduleName}`;
    return {
        artifact,
        ref,
    };
};
exports.resolveNPMSeed = resolveNPMSeed;
const getSeedDirs = (seed, dir, subdir, isInit, remote) => {
    const GENX_DIR = '/.genx';
    const TMP_DIR = '/tmp';
    const localPath = subdir ? `${seed}/${subdir}` : seed;
    const isLocalSeed = !remote && (0, node_fs_1.existsSync)(localPath);
    const genxDir = (0, node_path_1.relative)(process.cwd(), `${dir}${GENX_DIR}`);
    const extractedSeedDir = isInit ? dir : (0, node_path_1.relative)(process.cwd(), `${dir}${GENX_DIR}${TMP_DIR}`);
    const extractedSeedGenxDir = isInit
        ? genxDir
        : (0, node_path_1.relative)(process.cwd(), `${extractedSeedDir}${GENX_DIR}`);
    return {
        localPath,
        dir,
        subdir,
        isLocalSeed,
        genxDir,
        extractedSeedDir,
        extractedSeedGenxDir,
    };
};
exports.getSeedDirs = getSeedDirs;
const copyLocalSeed = (dirs, ref, start) => {
    const { dir, subdir, localPath, extractedSeedDir, genxDir, extractedSeedGenxDir } = dirs;
    consola_1.consola.info(`Copying local seed ${(0, console_1.bold)(localPath)}`);
    try {
        if (ref) {
            consola_1.consola.warn(`${(0, console_1.bold)('ref')} has no effect when used with a local seed`);
        }
        (0, node_fs_1.cpSync)(localPath, extractedSeedDir, { recursive: true });
        consola_1.consola.success(`Seed copied in ${Date.now() - start}ms`);
        return {
            local: true,
            genxDir,
            seed: localPath,
            subdir,
            pkg: (0, seed_meta_1.loadGenxModule)(extractedSeedGenxDir),
            dir,
            extractedSeedDir,
            extractedSeedGenxDir,
        };
    }
    catch (e) {
        consola_1.consola.error(`Error copying ${localPath} to ${(0, console_1.bold)(dir)}`, e);
        process.exit(1);
    }
};
const fetchNPMSeed = (seed, ref, dirs, useDefaultNPMScope, start) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    let success = true;
    const data = (0, exports.resolveNPMSeed)(seed, ref, useDefaultNPMScope);
    consola_1.consola.info(`Downloading NPM seed ${(0, console_1.bold)(data.artifact)} (dist tag ${(0, console_1.bold)(data.ref)}, subdir: ${(0, console_1.bold)(dirs.subdir || 'N/A')}) to ${(0, console_1.bold)(dirs.dir)} directory...`);
    try {
        const version = (0, npm_1.getLatestVersion)(data.artifact, data.ref, false);
        const url = (0, get_npm_tarball_url_1.default)(data.artifact, version);
        // NPM tarballs contains top-level directory called package
        const path = dirs.subdir ? `${node_path_1.sep}package/${dirs.subdir}` : `${node_path_1.sep}package`;
        yield (0, archive_1.downloadTarball)(url, dirs.extractedSeedDir, path).catch((err) => {
            consola_1.consola.warn('Error downloading module', err);
            success = false;
        });
    }
    catch (err) {
        success = false;
        consola_1.consola.warn('Error fetching version information from the registry');
    }
    // If NPM download successful return result, otherwise fall back to Git download
    if (success) {
        consola_1.consola.success(`Seed downloaded in ${Date.now() - start}ms`);
        return Object.assign({ local: false, genxDir: dirs.genxDir, seed, subdir: dirs.subdir, pkg: (0, seed_meta_1.loadGenxModule)(dirs.extractedSeedGenxDir), dir: dirs.dir, extractedSeedDir: dirs.extractedSeedDir, extractedSeedGenxDir: dirs.extractedSeedGenxDir }, data);
    }
});
const fetchGitSeed = (seed, ref, dirs, start) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const data = (0, exports.resolveGitSeed)(seed, ref, dirs.subdir);
    consola_1.consola.info(`Downloading Git seed ${(0, console_1.bold)(data.repo)} (Ref ${(0, console_1.bold)(data.ref)}, subdir: ${(0, console_1.bold)(dirs.subdir || 'N/A')}) to ${(0, console_1.bold)(dirs.dir)} directory...`);
    try {
        yield (0, giget_1.downloadTemplate)(data.id, {
            dir: dirs.extractedSeedDir,
            // auth?: string; // custom Authorization token to use for downloading template
            registry: false, // disable built-in Giget registry
        });
        consola_1.consola.success(`Seed downloaded in ${Date.now() - start}ms`);
        return Object.assign({ local: false, genxDir: dirs.genxDir, seed, subdir: dirs.subdir, pkg: (0, seed_meta_1.loadGenxModule)(dirs.extractedSeedGenxDir), dir: dirs.dir, extractedSeedDir: dirs.extractedSeedDir, extractedSeedGenxDir: dirs.extractedSeedGenxDir }, data);
    }
    catch (err) {
        const error = process.env.DEBUG ? err : err.message;
        consola_1.consola.error(error);
        process.exit(1);
    }
});
// download remote or copy local seed
const fetchSeed = (seed, dir, remote, ref, insecure, subdir, isInit, lookupInNPM, useDefaultNPMScope) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    var _a;
    // disable SSL cert verification
    if (insecure) {
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
    }
    const start = Date.now();
    const dirs = (0, exports.getSeedDirs)(seed, dir, subdir, isInit, remote);
    if (!isInit) {
        yield (0, fs_1.rm)(dirs.extractedSeedDir);
        (0, node_fs_1.mkdirSync)(dirs.extractedSeedDir);
    }
    let result;
    if (dirs.isLocalSeed) {
        result = copyLocalSeed(dirs, ref, start);
    }
    else {
        if (lookupInNPM) {
            result = yield fetchNPMSeed(seed, ref, dirs, useDefaultNPMScope, start);
        }
        // if NPM lookup disabled or if it's enabled & failed, fall back to Git
        if (!lookupInNPM || !result) {
            result = yield fetchGitSeed(seed, ref, dirs, start);
        }
    }
    yield (0, seed_processing_1.removeExclusions)(dirs.extractedSeedDir, (_a = result.pkg.genxSeedConfig) === null || _a === void 0 ? void 0 : _a.exclude);
    return result;
});
exports.fetchSeed = fetchSeed;
