"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureSeedApp = exports.installSeedDeps = void 0;
const tslib_1 = require("tslib");
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const changeCase = tslib_1.__importStar(require("change-case"));
const consola_1 = require("consola");
const child_process_1 = require("./child-process");
const fs_1 = require("./fs");
const handlebars_1 = require("./handlebars");
const json_1 = require("./json");
const object_1 = require("./object");
const prompts_1 = require("./prompts");
const seed_meta_1 = require("./seed.meta");
const seed_processing_1 = require("./seed.processing");
const installSeedDeps = (dir, isLogLevelVerbose) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    let dependencies = {};
    try {
        const packageJSONPath = (0, node_path_1.resolve)(dir, 'package.json');
        const packageJSON = require(packageJSONPath);
        dependencies = packageJSON.dependencies || {};
    }
    catch (e) { }
    consola_1.consola.debug('Seed NPM dependencies', {
        targetInstallDir: dir,
        dependencyCount: Object.keys(dependencies).length,
    });
    if (Object.keys(dependencies).length !== 0) {
        const start2 = Date.now();
        consola_1.consola.info('Installing seed NPM dependencies...');
        yield (0, child_process_1.runAsync)(dir, 'npm install', {
            prefixCommandWithNode: false,
            exitOnError: false,
            verbose: isLogLevelVerbose,
            env: Object.assign(Object.assign({}, process.env), { NODE_ENV: 'production' }),
        }).catch((err) => {
            consola_1.consola.error('Unexpected error installing seed dependencies. Try re-runing with --log-level verbose');
            if (err) {
                consola_1.consola.error(err);
            }
            process.exit(1);
        });
        consola_1.consola.success(`Installed in ${Date.now() - start2}ms`);
    }
});
exports.installSeedDeps = installSeedDeps;
// run seed configure script
const configureSeedApp = (seedInfo, fetchSeedResult, params, dir, skipOptionalPrompts, isInit) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const { configure, prompts, details, defaults, validate } = seedInfo;
    // load persisted answers if available
    const persistedAnswers = (0, seed_meta_1.loadGenxModule)(fetchSeedResult.genxDir, seed_processing_1.PROMPT_DATA_FILE, false);
    // get prompt answers
    const data = yield (0, prompts_1.getData)(persistedAnswers, prompts, details, defaults, validate, params, dir, skipOptionalPrompts, fetchSeedResult);
    consola_1.consola.debug('Loaded seed data', data);
    const utils = {
        writeFileWithData: handlebars_1.renderFile,
        registerPartial: handlebars_1.registerPartial,
        editJSONFile: json_1.editJSONFile,
        replaceObjectKeys: object_1.replaceObjectKeys,
        listAllFiles: fs_1.listAllFiles,
        rename: fs_1.rename,
        changeCase,
    };
    // run seed configure script
    yield configure(data, utils).catch((e) => consola_1.consola.error(e));
    // process seed exclusions, render templates, rename files/folders
    (0, seed_processing_1.processOptions)(fetchSeedResult, fetchSeedResult.extractedSeedDir, data, isInit, persistedAnswers);
    // when adding seed, copy result from temp folder to the app root (omitting .genx folder of the seed)
    if (!isInit) {
        yield (0, fs_1.rm)(fetchSeedResult.extractedSeedGenxDir);
        (0, node_fs_1.cpSync)(fetchSeedResult.extractedSeedDir, dir, { recursive: true });
        yield (0, fs_1.rm)(fetchSeedResult.extractedSeedDir);
    }
});
exports.configureSeedApp = configureSeedApp;
